@@ -155,25 +155,35 @@ def batch_import(self, batch_id, load_chunksize=5, placeholder_representation="%
         batch.save()
         logger.info('Begin batch %s' % (batch))
         
-        # DEPRECATED
-        """
-        for load in enqueued:
-            load.status = 'QUEUED'
-            load.save()
-            import_file(load.pk)
+        # decide now whether to use import_file in a chain (old way)
+        # or pg_import_file in a group (new hotness)
+        dsabbr = destination.source.abbreviation
         
-        return batch_complete.delay(batch.pk)
-        """
+        # FROM HERE BELOW, SPLIT OFF PSYCOPG2 VS NOT
         
-        # make a chain from task signatures that churn through each enqueued object
-        import_chain = celery.chain(
-            import_file2.si(load.pk) for load in enqueued
-            )
+        # check to see if psycopg2 is how we're going
+        db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
+        backend = db_def['ENGINE'].split('.')[-1]
         
-        # this could, alternatively, be a task group
+        if 'psycopg2' in backend and dsabbr.upper() in ['CORE', 'BASE']:
+            # do it Rockapella
+            job = celery.group(
+                pg_import_file.si(load.pk) for load in enqueued
+                )
+            
+            # fire the group
+            job()
         
-        # fire the chain
-        import_chain()
+        else:
+            # make a chain from task signatures that churn through each enqueued object
+            job = celery.chain(
+                import_file.si(load.pk) for load in enqueued
+                )
+            
+            # this could, alternatively, be a task group
+            
+            # fire the chain
+            job()
         
         # put a task into background for periodic check that the batch has completed
         batch_complete.apply_async((batch.pk,), countdown=60)
@@ -323,66 +333,124 @@ def import_file2(self, importqueue_pk, insert_size=5, slice_size=1000):
 
 @celery.shared_task(bind=True, ignore_result=True)
 def pg_import_file(self, importqueue_pk):
-    """Assumes file has been checked already for repeat import, meta matching, etc.
-    
-    Also assumes psycopg2 availability.
+    """Assumes psycopg2 availability.
     """
+    
     load = ImportQueue.objects.get(pk=importqueue_pk)
-    target_file = load.file
-    
-    c = target_file.file_type.compression
-    
-    if c == 'NONE':
-        # uncompressed
-        handle = open(target_file.full_path)
-    elif c == 'ZIP':
-        # pkzip compression
-        handle = pyhcup.hachoir.openz(os.path.dirname(target_file.full_path), target_file.filename)
     
+    load.celery_task_id = self.request.id
+    load.save()
     
     destination = load.batch.destination
-    meta = pyhcup.sas.meta_from_sas(target_file.loadfile.full_path)
-    table = destination.name
-    schema = destination.schema
-    state = target_file.state.abbreviation
-    year = target_file.year
+    similar_load_qs = ImportQueue.objects.filter(batch__destination=destination, file=load.file)
+    similar_completed = similar_load_qs.filter(complete__isnull=False)
+    similar_started = similar_load_qs.filter(start__isnull=False)
     
-    
-    # establish connection directly with psycopg2
-    import psycopg2
-    db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
-    cnxn = psycopg2.connect(
-        host=db_def['HOST'],
-        port=db_def['PORT'],
-        user=db_def['USER'],
-        password=db_def['PASSWORD'],
-        database=db_def['NAME'],
-        )
-    
-    # get helper functions
-    from pyhcup.db import pg_rawload, pg_staging, pg_shovel, pg_drop
-    
-    # let 'er rip
-    rawload = pg_rawload(cnxn, handle, meta)
-    staging = pg_staging(cnxn, rawload, meta, state, year)
-    
-    fields = [x for x in meta.field if x.lower() not in ['state', 'year']]
-    fields += ['state', 'year']
-    
-    affected_rows = pg_shovel(cnxn, staging[0], table, fields)
-    
-    pg_drop(rawload)
-    pg_drop(staging[0])
-    
-    # affected_rows will evaluate un-True in the case of any troubles
-    if affected_rows:
-        load.success()
-        logger.info('Import complete: %s (%s rows)' % (load, affected_rows))
-        return True
-    else:
+    if similar_completed.count() > 0:
+        #appears to already have been imported. skip this one
+        logger.warning('It looks like <%s> was already imported to <%s> at %s; skipping for now' % (load, destination, similar_completed[0].complete))
         load.fail()
-        logger.error('Import failed: %s' % load)
         return False
+    elif similar_started.count() > 0:
+        #appears to already have been imported. skip this one
+        logger.warning('It looks like an import was already started for importing <%s> to <%s> at %s; skipping for now' % (load, destination, similar_started[0].start))
+        load.fail()
+        return False
+    else:
+        # proceed with making the file happen
+        load.status = 'IN PROCESS'
+        load.start = datetime.datetime.utcnow().replace(tzinfo=utc)
+        load.save()
+        
+        logger.info('Begin import of %s to %s' % (load, destination))
+        
+        tls = destination.source.top_level_source.abbreviation
+        if tls == 'HCUP':
+            meta = pyhcup.sas.meta_from_sas(load.file.loadfile.full_path)
+            augmented = pyhcup.meta.augment(meta)
+            
+            # store the columns for future reference
+            for k, v in augmented.T.to_dict().iteritems():
+                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                col = Column(
+                    parent_file=load.file,
+                    name=v['field'],
+                    col_type=v['data_type'].upper(),
+                    col_scale=v['scale'],
+                    col_precision=v['length'],
+                    informat=v['informat'],
+                    )
+                col.save()
+        
+        elif tls == 'PUDF':
+            meta = pyhcup.tx.meta_from_txt(load.file.loadfile.full_path)
+            augmented = pyhcup.meta.augment(meta, dialect='PUDF')
+            
+            # store the columns for future reference
+            for k, v in augmented.T.to_dict().iteritems():
+                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                col = Column(
+                    parent_file=load.file,
+                    name=v['field'],
+                    col_type=v['data_type'].upper(),
+                    col_scale=v['scale'],
+                    col_precision=v['length'],
+                    # informat=v['informat'],
+                    )
+                col.save()
+                
+        table = destination.name
+        schema = destination.schema
+        state = load.file.state.abbreviation
+        year = load.file.year
+        dsabbr = destination.source.abbreviation
+
+        target_file = load.file
+        
+        c = target_file.file_type.compression
+        
+        if c == 'NONE':
+            # uncompressed
+            handle = open(target_file.full_path)
+        elif c == 'ZIP':
+            # pkzip compression
+            handle = pyhcup.hachoir.openz(os.path.dirname(target_file.full_path), target_file.filename)    
+        
+        # establish connection directly with psycopg2
+        import psycopg2
+        db_def = settings.DATABASES[HACHOIR_DB_ENTRY]
+        cnxn = psycopg2.connect(
+            host=db_def['HOST'],
+            port=db_def['PORT'],
+            user=db_def['USER'],
+            password=db_def['PASSWORD'],
+            database=db_def['NAME'],
+            )
+        
+        # get helper functions
+        from pyhcup.db import pg_rawload, pg_staging, pg_shovel, pg_drop
+        
+        # let 'er rip
+        rawload = pg_rawload(cnxn, handle, meta)
+        staging = pg_staging(cnxn, rawload, meta, state, year)
+        
+        fields = [x for x in meta.field if x.lower() not in ['state', 'year']]
+        fields += ['state', 'year']
+        
+        affected_rows = pg_shovel(cnxn, staging[0], table, fields)
+        
+        pg_drop(cnxn, rawload)
+        pg_drop(cnxn, staging[0])
+        
+        # affected_rows will evaluate un-True in the case of any troubles
+        if affected_rows:
+            load.success()
+            logger.info('Import complete: %s (%s rows)' % (load, affected_rows))
+            return True
+        else:
+            load.fail()
+            logger.error('Import failed: %s' % load)
+            return False
 
 
 @celery.shared_task(bind=True, ignore_result=True)
