@@ -390,9 +390,7 @@ def src_gen_master_table(request, source_id):
         
     try:
         cursor.execute(create_sql)
-        print "cursor.execute() without error"
         tx.commit(using=DJHCUP_DB_ENTRY)
-        print "tx.commit() without error"
     except:
         e = sys.exc_info()[0]
         raise Exception("Failed to create master table with query %s (%s)" % (create_sql, e))
@@ -409,7 +407,7 @@ def src_gen_master_table(request, source_id):
         except:
             raise Exception("Failed to create indexes on table, particularly one that should have resulted from this query: %s" % index)
     
-    table_record = ImportTable(source=source, name=db_table, database=DJHCUP_DB_ENTRY, schema=settings[DJHCUP_DB_ENTRY]['schema'])
+    table_record = ImportTable(source=source, name=db_table, database=DJHCUP_DB_ENTRY)
     table_record.save()
     logger.info('Saved master table details as ImportTable record %d' % table_record.pk)
     
